/**
 * @file
 * Open Rose JS.
 */

(function ($) {
  "use strict";
  Drupal.openy_rose = Drupal.openy_rose || {};
  Drupal.behaviors.openy_rose_theme = {
    attach: function (context, settings) {
      $('.ui-tabs').tabs({
        active: false,
        collapsible: true
      });
    }
  };

  // Sidebar collapsible.
  Drupal.behaviors.sidebar = {
    attach: function (context, settings) {
      var current_scroll = 0;
      $('.sidebar')
        .once()
        .on('show.bs.collapse',
          // Add custom class for expand specific styling. in = open.
          function (e) {
            $(this)
              .next('.viewport')
              .addBack()
              .removeClass('out')
              .addClass('collapsing-in')
              .removeAttr('aria-hidden');

            current_scroll = $(window).scrollTop();
            $('.nav-global').css({
              top: current_scroll
            });
          }
        )
        .on('shown.bs.collapse',
          // Allow css to control open rest state.
          function () {
            $(this)
              .next('.viewport')
              .addBack()
              .removeClass('collapsing-in')
              .addClass('in')
              .removeAttr('aria-hidden');

            var body = $('body');

            body.addClass('sidebar-in');

            $('html').addClass('sidebar-in');
          }
        )
        .on('hide.bs.collapse',
          // Add custom class for collapse specific styling. out = closed.
          function (e) {
            var sidebar = $(this);
            sidebar
              .next('.viewport')
              .addBack()
              .removeClass('in')
              .addClass('collapsing-out')
              .attr('aria-hidden', 'true');


            $(window).scrollTop(current_scroll);

            $('#page-head').css({
              marginTop: ''
            });

          }
        )
        .on('hidden.bs.collapse',
          // Allow css to control closed rest state.
          function () {
            $(this)
              .next('.viewport')
              .addBack()
              .addClass('out')
              .removeClass('collapsing-out');

            $('body').removeClass('sidebar-in');
            $('html').removeClass('sidebar-in');

            $('.nav-global').css({
              top: 0
            });
          }
        )
        .find('li')
        .on('hide.bs.dropdown',
          // For nested dropdowns, prevent collapse of other dropdowns.
          function (e) {
            e.preventDefault();
          }
        );
    }
  };

  // Sidebar collapsible menu items.
  Drupal.behaviors.sidebarMenuCollapsible = {
    attach: function (context, settings) {
      $('.sidebar .dropdown-toggle').on('click', function () {
        var expanded = $(this).attr('aria-expanded');
        if (expanded == 'true') {
          $(this).removeAttr('aria-expanded');
          $(this).parent().removeClass('open');
          return false;
        }
      });
    }
  };

  // Horizontal scroll for camp menu.
  Drupal.behaviors.scrollableList = {
    attach: function (context, settings) {
      $('.camp-menu-wrapper', context).once().each(function () {
        var $this = $(this),
            $list = $this.find('ul'),
            $items = $list.find('li'),
            listWidth = 0,
            listPadding = 40;

        setTimeout(function () {
          $items.each(function () {
            listWidth += $(this).outerWidth();
          });

          $list.css('width', listWidth + listPadding + "px");

          var columns = $this.find('.columns');
          if (columns.length == 0) {
            return;
          }
          var scroll = new IScroll(columns[0], {
            scrollX: true,
            scrollY: false,
            momentum: false,
            snap: false,
            bounce: true,
            touch: true,
            eventPassthrough: true
          });

          // GRADIENT BEHAVIOUR SCRIPT.
          var obj = $('.camp-menu');
          var objWrap = columns.append('<div class="columns-gradient gradient-right" onclick="void(0)"></div>');
          objWrap = document.querySelector('.columns-gradient');
          var sliderLength = listWidth - objWrap.offsetWidth + 40;
          var firstGap = 20;

          if (window.innerWidth > 768) {
            sliderLength = listWidth - objWrap.offsetWidth + 150;
            firstGap = 60;
          }

          obj.get(0).addEventListener('touchmove', function () {
            var transformMatrix = obj.css("-webkit-transform") ||
                obj.css("-moz-transform")    ||
                obj.css("-ms-transform")     ||
                obj.css("-o-transform")      ||
                obj.css("transform");
            var matrix = transformMatrix.replace(/[^0-9\-.,]/g, '').split(',');
            var x = matrix[12] || matrix[4];
            var y = matrix[13] || matrix[5];
            console.log(x, y);
            if (x <= -sliderLength + listPadding) {
              objWrap.classList.remove('gradient-right');
            }
            else {
              objWrap.classList.add('gradient-right');
            }

            if (x >= -firstGap) {
              objWrap.classList.remove('gradient-left');
            }
            else {
              objWrap.classList.add('gradient-left');
            }
          });
        }, 100);
      });
    }
  };

  // Adjust labels for hamburger menu icon.
  Drupal.behaviors.menuIconLabelChange = {
    attach: function (context, settings) {
      $('.navbar-toggle').on('click', function () {
        if ($(this).attr('aria-expanded') == 'false') {
          $(this).children('.sr-only').text(Drupal.t('Close main navigation'));
        } else {
          $(this).children('.sr-only').text(Drupal.t('Navigation menu'));
        }
      });
    }
  };

  /**
   * Adjust the top nav position when the skip link is in focus.
   */
  Drupal.behaviors.adjustSkipLink = {
    attach: function (context, settings) {
      // On focus, move the top nav down to show the skip link.
      $('.skip-link').on('focus', function () {
        var link_height = $(this).height();
        $('.top-navs').css({'margin-top': link_height});
      });
      // When focus is lost, remove the unneeded height.
      $('.skip-link').on('focusout', function () {
        $('.top-navs').css({'margin-top': '0'});
      });
    }
  };

  /**
   * Add focus for first loaded element.
   */
  Drupal.behaviors.load_more_focus = {
    attach: function (context, settings) {
      $('.views-element-container .load_more_button .button', context).click(function () {
        var $viewsRow = $('.views-element-container .views-row'),
          indexLastRow = $viewsRow.length,
          getElement,
          itemFocus;
        if (Drupal.views !== undefined) {
          $.each(Drupal.views.instances, function (i, view) {
            if (view.settings.view_name.length != 0) {
              $(document).ajaxComplete(function (event, xhr, settings) {
                getElement = $('.views-element-container .views-row');
                itemFocus = getElement[indexLastRow];
                // Add focus to element.
                $(itemFocus).find('h3 a').focus();
                // Update number indexLastRow.
                $viewsRow = $('.views-element-container .views-row');
                indexLastRow = $viewsRow.length;
              });
            }
          });
        }
      });
    }
  };

})(jQuery);
